#ifndef __GEOMETRY_H__
#define __GEOMETRY_H__

#include <QVector3D>
#include <QtOpenGL>
#include <GL/glu.h>
#include <Aabb.h>

class Mesh;

/**
 * Geometry functions
 * @author Stphane GINIER
 */
namespace Geometry
{
    bool intersectionRayTriangle(const QVector3D &s1, const QVector3D &s2, const QVector3D &v1,
                      const QVector3D &v2, const QVector3D &v3, const QVector3D &normal,
                      QVector3D &vertInter);
    QVector3D intersectionRayPlane(const QVector3D &s1, const QVector3D &s2, const QVector3D &v1, const QVector3D &normal);
    QVector3D mouseOnUnitSphere(const QPointF &mouseXY);
    QPointF normalizedMouse(const QPointF &mouseXY, const QSizeF window);
    void getRayNormalToScreen(const QPointF &mouseXY, const QSizeF &window,
                                  QVector3D &vertexNear, QVector3D &vertexFar);
    QVector3D getCoplanarVectorOnScreen(const QPointF &mouseXYOld, const QPointF &mouseXYNew, const QSizeF &viewport);
    QVector3D vertexOnLine(const QVector3D &vertex, const QVector3D &vertexNear, const QVector3D &vertexFar);
    QVector3D point2Dto3D(const QPointF &mouseXY,const QSizeF &viewport, float z=0.0f);
    QPointF point3Dto2D(const QVector3D &vertex,const QSizeF &viewport, double &z);
    QVector3D getPerpendicularVector(const QVector3D &vec);
    Aabb computeTriangleAabb(const QVector3D &v1, const QVector3D &v2, const QVector3D &v3);
    float computeInradiusSquared(const QVector3D &v1, const QVector3D &v2, const QVector3D &v3);
    bool pointInsideTriangle(const QVector3D &point, const QVector3D &v1, const QVector3D &v2, const QVector3D &v3);
    bool sphereIntersectTriangle(const QVector3D &point, float radiusSq, const QVector3D &v1, const QVector3D &v2, const QVector3D &v3);
    float distanceToSegment(const QVector3D &point, const QVector3D &v1, const QVector3D &v2);
}

#endif /*__GEOMETRY_H__*/
