#ifndef __AABB_H__
#define __AABB_H__

#include <QVector3D>
#include <QtOpenGL>
#include "Vertex.h"

/**
 * Axis Aligned Bounding Box (AABB)
 * @author Stphane GINIER
 */
class Aabb
{
public:
    Aabb(const QVector3D &min = QVector3D(), const QVector3D &max = QVector3D());
    ~Aabb();
    QVector3D getCenter() const;

    bool pointInside(const QVector3D &vert) const;
    bool isOutside(const Aabb &aabb) const;
    bool isInside(const Aabb &aabb)const;
    void expand(const QVector3D &vert);
    void expand(const Aabb &aabb);
    bool intersectRay(const QVector3D &vert, const QVector3D &dir) const;
    bool intersectSphere(const QVector3D &vert, float radiusSquared) const;
    void checkFlat(float offset);
    void draw(const QColor &color) const;

public :
    QVector3D min_; //min vertex
    QVector3D max_; //max vertex
};

#endif /*__AABB_H__*/
